@@ -184,7 +184,7 @@ int main(void)
   rv = curl_easy_setopt(ch,CURLOPT_WRITEFUNCTION, *writefunction);
   rv = curl_easy_setopt(ch,CURLOPT_WRITEDATA, stdout);
   rv = curl_easy_setopt(ch,CURLOPT_HEADERFUNCTION, *writefunction);
-  rv = curl_easy_setopt(ch,CURLOPT_WRITEHEADER, stderr);
+  rv = curl_easy_setopt(ch,CURLOPT_HEADERDATA, stderr);
   rv = curl_easy_setopt(ch,CURLOPT_SSLCERTTYPE,"PEM");
 
   /* both VERIFYPEER and VERIFYHOST are set to 0 in this case because there is
